@@ -3,25 +3,18 @@ package Gscan2pdf::Dialog;
 use warnings;
 use strict;
 use Gtk3;
-use Glib 1.220 qw(TRUE FALSE);    # To get TRUE and FALSE
+use Glib 1.220 qw(TRUE FALSE);      # To get TRUE and FALSE
+use Gscan2pdf::PageRange;
+use Gscan2pdf::Translation '__';    # easier to extract strings with xgettext
 use Data::Dumper;
 $Data::Dumper::Sortkeys = 1;
 
-use Glib::Object::Subclass Gtk3::Window::,
+use Glib::Object::Subclass Gtk3::Dialog::,
   signals => {
     delete_event    => \&on_delete_event,
     key_press_event => \&on_key_press_event,
   },
   properties => [
-    Glib::ParamSpec->uint(
-        'border-width',             # name
-        'Border width',             # nickname
-        'Border width for vbox',    # blurb
-        0,                          # min
-        999,                        # max
-        0,                          # default
-        [qw/readable writable/]     # flags
-    ),
     Glib::ParamSpec->boolean(
         'hide-on-delete',                                             # name
         'Hide on delete',                                             # nickname
@@ -29,16 +22,17 @@ use Glib::Object::Subclass Gtk3::Window::,
         FALSE,                                                        # default
         [qw/readable writable/]                                       # flags
     ),
-    Glib::ParamSpec->object(
-        'vbox',                                                       # name
-        'VBox',                                                       # nickname
-        'VBox which is automatically added to the Gscan2pdf::Dialog', # blurb
-        'Gtk3::VBox',                                                 # package
-        [qw/readable writable/]                                       # flags
+    Glib::ParamSpec->enum(
+        'page-range',                                                 # name
+        'page-range',                                                 # nickname
+        'Either selected or all',                                     #blurb
+        'Gscan2pdf::PageRange::Range',
+        'selected',                                                   # default
+        [qw/readable writable/]                                       #flags
     ),
   ];
 
-our $VERSION = '2.1.4';
+our $VERSION = '2.1.6';
 my $EMPTY    = q{};
 my $HEXREGEX = qr{^(.*)           # start of message
                   \b0x[[:xdigit:]]+\b # hex (e.g. address)
@@ -51,26 +45,10 @@ my $INTREGEX = qr{^(.*)           # start of message
 
 sub INIT_INSTANCE {
     my $self = shift;
-
     $self->set_position('center-on-parent');
-
-    # VBox for window
-    my $vbox = Gtk3::VBox->new;
-    $self->add($vbox);
-    $self->set( 'vbox', $vbox );
     return $self;
 }
 
-sub SET_PROPERTY {
-    my ( $self, $pspec, $newval ) = @_;
-    my $name = $pspec->get_name;
-    $self->{$name} = $newval;
-    if ( $name eq 'border_width' ) {
-        $self->get('vbox')->set( 'border-width', $newval );
-    }
-    return;
-}
-
 sub on_delete_event {
     my ( $widget, $event ) = @_;
     if ( $widget->get('hide-on-delete') ) {
@@ -96,6 +74,42 @@ sub on_key_press_event {
     return Gtk3::EVENT_STOP;
 }
 
+# Add a frame and radio buttons to $vbox,
+sub add_page_range {
+    my ($self) = @_;
+    my $frame = Gtk3::Frame->new( __('Page Range') );
+    $self->get_content_area->pack_start( $frame, FALSE, FALSE, 0 );
+
+    my $pr = Gscan2pdf::PageRange->new;
+    $pr->set_active( $self->get('page-range') );
+    $pr->signal_connect(
+        changed => sub {
+            $self->set( 'page-range', $pr->get_active );
+        }
+    );
+    $frame->add($pr);
+    return;
+}
+
+# Add buttons and link up their actions
+sub add_actions {
+    my ( $self, $button1, $callback1, $button2, $callback2 ) = @_;
+    $self->add_buttons( $button1 => 'ok', $button2 => 'cancel' );
+    $self->set_default_response('ok');
+    $self->signal_connect(
+        response => sub {
+            my ( $widget, $response ) = @_;
+            if ( $response eq 'ok' ) {
+                $callback1->();
+            }
+            else {
+                $callback2->();
+            }
+        }
+    );
+    return;
+}
+
 sub dump_or_stringify {
     my ($val) = @_;
     return (
