@@ -69,7 +69,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.1.4';
+our $VERSION = '2.1.6';
 
 my ( $start_old, $step_old, $logger );
 
@@ -87,7 +87,7 @@ sub new {
 
     $self->set( 'title', __('Renumber') );
 
-    my $vbox = $self->get('vbox');
+    my $vbox = $self->get_content_area;
 
     # Frame for page range
     my $frame = Gtk3::Frame->new( __('Page Range') );
@@ -112,7 +112,7 @@ sub new {
     my $framex = Gtk3::Frame->new( __('Page numbering') );
     $vbox->pack_start( $framex, FALSE, FALSE, 0 );
     my $vboxx = Gtk3::VBox->new;
-    $vboxx->set_border_width( $self->get('border_width') );
+    $vboxx->set_border_width( $self->style_get('content-area-border') );
     $framex->add($vboxx);
 
     # SpinButton for starting page number
@@ -183,19 +183,10 @@ sub new {
         }
     );
 
-    # HBox for buttons
-    my $hbox = Gtk3::HBox->new;
-    $vbox->pack_start( $hbox, FALSE, TRUE, 0 );
-
-    # Start button
-    my $obutton = Gtk3::Button->new( __('Renumber') );
-    $hbox->pack_start( $obutton, TRUE, TRUE, 0 );
-    $obutton->signal_connect( clicked => sub { $self->renumber } );
-
-    # Close button
-    my $cbutton = Gtk3::Button->new_from_stock('gtk-close');
-    $hbox->pack_end( $cbutton, FALSE, FALSE, 0 );
-    $cbutton->signal_connect( clicked => sub { $self->hide; } );
+    $self->add_actions(
+        __('Renumber'), sub { $self->renumber },
+        'gtk-close',    sub { $self->hide }
+    );
 
     return $self;
 }
