@@ -9,13 +9,12 @@ use Data::Dumper;
 $Data::Dumper::Sortkeys = 1;
 use Storable qw(dclone);
 use feature 'switch';
+use Gscan2pdf::ComboBoxText;
 use Gscan2pdf::Dialog;
 use Gscan2pdf::Scanner::Options;
 use Gscan2pdf::Scanner::Profile;
 use Gscan2pdf::Translation '__';    # easier to extract strings with xgettext
 use List::MoreUtils qw(first_index);
-use Readonly;
-Readonly my $BORDER_WIDTH => 6;
 
 my (
     $MAX_PAGES,        $MAX_INCREMENT, $DOUBLE_INCREMENT,
@@ -268,7 +267,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.1.2';
+our $VERSION = '2.1.6';
 
 my ( $d_sane, $logger );
 my $tolerance             = 1;
@@ -282,7 +281,7 @@ my $SANE_NAME_PAGE_WIDTH  = SANE_NAME_PAGE_WIDTH;
 sub INIT_INSTANCE {
     my $self = shift;
 
-    my $vbox = $self->get('vbox');
+    my $vbox = $self->get_content_area;
 
     $d_sane = Locale::gettext->domain('sane-backends');
 
@@ -299,14 +298,14 @@ sub INIT_INSTANCE {
     $scwin->set_policy( 'automatic', 'automatic' );
     my $vbox1 = Gtk3::VBox->new;
     $self->{vbox} = $vbox1;
-    $vbox1->set_border_width($BORDER_WIDTH);
+    my $border_width = $self->style_get('content-area-border');
+    $vbox1->set_border_width($border_width);
     $scwin->add_with_viewport($vbox1);
 
     # Frame for # pages
     $self->{framen} = Gtk3::Frame->new( __('# Pages') );
     $vbox1->pack_start( $self->{framen}, FALSE, FALSE, 0 );
-    my $vboxn        = Gtk3::VBox->new;
-    my $border_width = $self->get('border_width');
+    my $vboxn = Gtk3::VBox->new;
     $vboxn->set_border_width($border_width);
     $self->{framen}->add($vboxn);
 
@@ -524,7 +523,7 @@ sub INIT_INSTANCE {
     my $vboxsp = Gtk3::VBox->new;
     $vboxsp->set_border_width($border_width);
     $framesp->add($vboxsp);
-    $self->{combobsp} = Gtk3::ComboBoxText->new;
+    $self->{combobsp} = Gscan2pdf::ComboBoxText->new;
     $self->{combobsp}->signal_connect(
         changed => sub {
             $self->{num_reloads} = 0;    # num-reloads is read-only
@@ -549,25 +548,15 @@ sub INIT_INSTANCE {
     );
     $hboxsp->pack_start( $dbutton, FALSE, FALSE, 0 );
 
-    # HBox for buttons
-    my $hboxb = Gtk3::HBox->new;
-    $vbox->pack_end( $hboxb, FALSE, FALSE, 0 );
-
-    # Scan button
-    $self->{sbutton} = Gtk3::Button->new( __('Scan') );
-    $hboxb->pack_start( $self->{sbutton}, TRUE, TRUE, 0 );
-    $self->{sbutton}->signal_connect(
-        clicked => sub {
+    $self->add_actions(
+        __('Scan'),
+        sub {
             $self->signal_emit('clicked-scan-button');
             $self->scan;
-        }
+        },
+        'gtk-close',
+        sub { $self->hide; }
     );
-    $self->{sbutton}->grab_focus;
-
-    # Cancel button
-    my $cbutton = Gtk3::Button->new_from_stock('gtk-close');
-    $hboxb->pack_end( $cbutton, FALSE, FALSE, 0 );
-    $cbutton->signal_connect( clicked => sub { $self->hide; } );
 
     # initialise stack of uuids - needed for cases where setting a profile
     # requires several reloads, and therefore reapplying the same profile
@@ -583,7 +572,7 @@ sub _add_device_combobox {
     $self->{hboxd} = Gtk3::HBox->new;
     my $labeld = Gtk3::Label->new( __('Device') );
     $self->{hboxd}->pack_start( $labeld, FALSE, FALSE, 0 );
-    $self->{combobd} = Gtk3::ComboBoxText->new;
+    $self->{combobd} = Gscan2pdf::ComboBoxText->new;
     $self->{combobd}->append_text( __('Rescan for devices') );
 
     $self->{combobd_changed_signal} = $self->{combobd}->signal_connect(
@@ -609,8 +598,7 @@ sub _add_device_combobox {
             if ( defined $device and $device ne $EMPTY ) {
                 for ( @{$device_list} ) {
                     if ( $_->{name} eq $device ) {
-                        Gscan2pdf::Dialog::Scan::set_combobox_by_text(
-                            $self->{combobd}, $_->{label} );
+                        $self->{combobd}->set_active_by_text( $_->{label} );
                         $self->scan_options($device);
                         return;
                     }
@@ -771,7 +759,7 @@ sub SET_PROPERTY {
                     'changed-paper' => sub {
                         $self->signal_handler_disconnect($signal);
                         my $paper = defined $newval ? $newval : __('Manual');
-                        set_combobox_by_text( $self->{combobp}, $paper );
+                        $self->{combobp}->set_active_by_text($paper);
                         if ( defined $logger ) {
                             $logger->debug("Finished$msg");
                         }
@@ -792,7 +780,7 @@ sub SET_PROPERTY {
                 $signal = $self->signal_connect(
                     'changed-profile' => sub {
                         $self->signal_handler_disconnect($signal);
-                        set_combobox_by_text( $self->{combobsp}, $newval );
+                        $self->{combobsp}->set_active_by_text($newval);
                         if ( defined $logger ) {
                             $logger->debug("Finished$msg");
                         }
@@ -805,7 +793,7 @@ sub SET_PROPERTY {
                 $self->signal_emit( 'changed-option-visibility', $newval );
             }
             default {
-                $self->SUPER::SET_PROPERTY( $pspec, $newval );
+                $self->{$name} = $newval;
             }
         }
         if ( defined $logger and not $callback ) {
@@ -1006,7 +994,7 @@ sub set_device_list {
     $self->{combobd}->signal_handler_block( $self->{combobd_changed_signal} );
 
     # Remove all entries apart from rescan
-    my $num_rows = get_combobox_num_rows( $self->{combobd} );
+    my $num_rows = $self->{combobd}->get_num_rows;
     while ( $num_rows-- > 1 ) {
         $self->{combobd}->remove(0);
     }
@@ -1110,7 +1098,7 @@ sub create_paper_widget {
         my $label = Gtk3::Label->new( __('Paper size') );
         $hboxp->pack_start( $label, FALSE, FALSE, 0 );
 
-        $self->{combobp} = Gtk3::ComboBoxText->new;
+        $self->{combobp} = Gscan2pdf::ComboBoxText->new;
         $self->{combobp}->append_text( __('Manual') );
         $self->{combobp}->append_text( __('Edit') );
         $self->{combobp}
@@ -1368,7 +1356,7 @@ sub set_paper_formats {
     if ( defined $combobp ) {
 
         # Remove all formats, leaving Manual and Edit
-        my $n = get_combobox_num_rows($combobp);
+        my $n = $combobp->get_num_rows;
         while ( $n-- > 2 ) { $combobp->remove(0) }
 
         $self->{ignored_paper_formats} = ();
@@ -1387,7 +1375,7 @@ sub set_paper_formats {
         # Set the combobox back from Edit to the previous value
         my $paper = $self->get('paper');
         if ( not defined $paper ) { $paper = __('Manual') }
-        set_combobox_by_text( $combobp, $paper );
+        $combobp->set_active_by_text($paper);
     }
     return;
 }
@@ -1480,9 +1468,8 @@ sub edit_paper {
     my $window = Gscan2pdf::Dialog->new(
         'transient-for' => $self,
         title           => __('Edit paper size'),
-        border_width    => $self->get('border-width'),
     );
-    my $vbox = $window->get('vbox');
+    my $vbox = $window->get_content_area;
 
     # Buttons for SimpleList
     my $hboxl = Gtk3::HBox->new;
@@ -1638,7 +1625,7 @@ sub edit_paper {
         clicked => sub {
 
             # Set the combobox back from Edit to the previous value
-            set_combobox_by_text( $combobp, $self->get('paper') );
+            $combobp->set_active_by_text( $self->get('paper') );
 
             $window->destroy;
         }
@@ -1652,8 +1639,7 @@ sub edit_paper {
 sub save_current_profile {
     my ( $self, $name ) = @_;
     $self->add_profile( $name, $self->{current_scan_options} );
-    $self->{combobsp}
-      ->set_active( get_combobox_num_rows( $self->{combobsp} ) - 1 );
+    $self->{combobsp}->set_active( $self->{combobsp}->get_num_rows - 1 );
     return;
 }
 
@@ -1677,7 +1663,7 @@ sub add_profile {
     # we get strange action-at-a-distance problems
     $self->{profiles}{$name} = dclone($profile);
 
-    _combobox_remove_item_by_text( $self->{combobsp}, $name );
+    $self->{combobsp}->remove_item_by_text($name);
 
     $self->{combobsp}->append_text($name);
     $logger->debug( "Saved profile '$name':",
@@ -1686,20 +1672,6 @@ sub add_profile {
     return;
 }
 
-sub _combobox_remove_item_by_text {
-    my ( $combobox, $text ) = @_;
-    if ( defined $text ) {
-        my $i = get_combobox_by_text( $combobox, $text );
-        if ( $i > $NO_INDEX ) {
-            if ( $combobox->get_active == $i ) {
-                $combobox->set_active($NO_INDEX);
-            }
-            $combobox->remove($i);
-        }
-    }
-    return;
-}
-
 sub set_profile {
     my ( $self, $name ) = @_;
     if ( defined $name and $name ne $EMPTY ) {
@@ -1747,64 +1719,13 @@ sub set_profile {
 sub remove_profile {
     my ( $self, $name ) = @_;
     if ( defined $name and defined $self->{profiles}{$name} ) {
-        _combobox_remove_item_by_text( $self->{combobsp}, $name );
+        $self->{combobsp}->remove_item_by_text($name);
         $self->signal_emit( 'removed-profile', $name );
         delete $self->{profiles}{$name};
     }
     return;
 }
 
-sub get_combobox_num_rows {
-    my ($combobox) = @_;
-    my $i = 0;
-    if ( defined $combobox and defined( $combobox->get_model ) ) {
-        $combobox->get_model->foreach(
-            sub {
-                ++$i;
-                return FALSE;    # continue the foreach()
-            }
-        );
-    }
-    return $i;
-}
-
-sub get_combobox_by_text {
-    my ( $combobox, $text ) = @_;
-    my $o = $NO_INDEX;
-    my $i = 0;
-    if (    defined $combobox
-        and defined( $combobox->get_model )
-        and defined $text )
-    {
-        $combobox->get_model->foreach(
-            sub {
-                my ( $model, $path, $iter ) = @_;
-                if ( $model->get( $iter, 0 ) eq $text ) {
-                    $o = $i;
-                    return TRUE;    # found - stop the foreach()
-                }
-                else {
-                    ++$i;
-                    return FALSE;    # not found - continue the foreach()
-                }
-            }
-        );
-    }
-    return $o;
-}
-
-sub set_combobox_by_text {
-    my ( $combobox, $text ) = @_;
-    if ( defined $combobox ) {
-        my $index = get_combobox_by_text( $combobox, $text );
-        if ( $index > $NO_INDEX or not defined $text ) {
-            $combobox->set_active($index);
-            return TRUE;
-        }
-    }
-    return;
-}
-
 sub _extended_pagenumber_checkbox_callback {
     my ( $widget, $param, $data ) = @_;
     my ( $dialog, $spin_buttoni ) = @{$data};
@@ -1878,7 +1799,6 @@ sub set_options {
         'transient-for' => $self,
         title           => $d_sane->get( $opt->{title} ),
         destroy         => TRUE,
-        border_width    => $self->get('border_width'),
     );
     my $vbox   = $window->vbox;
     my $canvas = Goo::Canvas->new;
